from functools import reduce
import operator

class deep_nest(object):
    def __init__(self, input_dict=dict()):
        self.__plain_dict = input_dict.copy()

    def __query_nested_dict(self, dataDict, mapList=[]):
        if mapList:
            return reduce(operator.getitem, mapList, dataDict)
        else:
            return self.__plain_dict

    def __getitem__(self, keys):
        if type(keys) is not tuple: keys = (keys, )
        try:
            return self.__query_nested_dict(self.__plain_dict, keys[:-1])[keys[-1]]
        except:
            return None

    def __setitem__(self, keys, value):
        if type(keys) is not tuple: keys = (keys, )
        try:
            self.__query_nested_dict(self.__plain_dict, keys[:-1])[keys[-1]] = value
        except (KeyError, TypeError) as e:
            tmp = reduce(lambda x, y: dict({y: x}), reversed(list(keys)), value)
            self.__plain_dict.update(tmp)

    def __deep_delete__(self, keys, dict=None):
        if not dict: dict = self.__plain_dict
        if type(keys) is not tuple: keys = (keys,)
        if len(keys) > 1:
            empty = self.__deep_delete__(dict=dict[keys[0]], keys=keys[1:])
            if empty:
                del dict[keys[0]]
        else:
            del dict[keys[0]]
        return len(dict) == 0

    def __delitem__(self, keys, dict=None):
        try:
            self.__deep_delete__(keys, dict=dict)
        except:
            return None

    def __repr__(self):
        return str(self.__plain_dict)

    def keys(self):
        return self.__plain_dict.keys()


from unittest import TestCase, skip
from deep_nest import deep_nest
__author__ = 'L. Ruhlen'
__project__ = 'deep_nest'

class TestDeepNest(TestCase):
    def setUp(self):
        basic_dict = {1: {'A': 'value'},
                      2: {'A': {'i': 'value'},
                          'B':'value'
                          },
                      3: {'A': 'value'}
                      }
        self.test_dict = deep_nest(basic_dict)

    def test_get_item(self):
        tmp = self.test_dict[1, 'A']
        self.assertEquals(tmp, 'value')

        tmp = self.test_dict[2, 'A']
        self.assertEquals(tmp, {'i': 'value'})

        tmp = self.test_dict[2]
        self.assertEquals(tmp, {'A': {'i': 'value'}, 'B':'value'} )

    def test_set_item(self):
        self.test_dict[3,'A'] = 'MOOSE'
        tmp = self.test_dict[3,'A']
        self.assertEquals(tmp, 'MOOSE')

    def test_delete_item(self):
        del self.test_dict[2,'A']
        self.assertEquals(self.test_dict[2], {'B': 'value'})

        del self.test_dict[2, 'B', 'NOT A REAL KEY']
        self.assertEquals(self.test_dict[2], {'B': 'value'})

    def test_insert_new_item(self):
        self.test_dict[1, 'B'] = 'TEST_VAL'
        self.assertEquals(self.test_dict[1,'B'], 'TEST_VAL')

        self.test_dict['BRAND NEW KEY', 12, 'RANDOM STRING'] = 93
        self.assertEquals(self.test_dict['BRAND NEW KEY', 12, 'RANDOM STRING'], 93)

        self.test_dict[3, 'A', 'value'] = 20
        self.assertEquals(self.test_dict[3, 'A', 'value'], 20)

    def test_print_ndict(self):
        print(self.test_dict)
        pass




from setuptools import setup

setup(name='deep_nest',
      version='0.12',
      description='A sane approach to arbitrarily nested dictionaries (*cough* nested JSON *cough*)',
      url='https://github.com/lruhlen/deep_nest',
      author='L. Ruhlen',
      author_email='ruhlen@gmail.com',
      license='MIT',
      packages=['deep_nest'],
      install_requires=[],
      zip_safe=False)
